<?php

/**
 * @file
 * Contains \Drupal\field_ui_ajax\Form\FieldConfigAjaxEditForm.
 */

namespace Drupal\field_ui_ajax\Form;

use Drupal\Core\Entity\EntityForm;
use Drupal\field_ui\Form\FieldConfigEditForm;
use Drupal\Core\Field\AllowedTagsXssTrait;
use Drupal\Core\Field\FieldFilteredMarkup;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;
use Drupal\field\FieldConfigInterface;
use Drupal\field_ui\FieldUI;
use Drupal\Core\Ajax\AjaxResponse;
use Drupal\Core\Ajax\HtmlCommand;
use Drupal\Core\Ajax\InvokeCommand;
use Drupal\Core\Ajax\OpenDialogCommand;
use Drupal\field_ui_ajax\Component\Utility\HtmlExtra;

/**
 * Provides a form for the field settings form.
 */
class FieldConfigAjaxEditForm extends FieldConfigEditForm {

  /**
   * {@inheritdoc}
   */
  public function form(array $form, FormStateInterface $form_state) {
    $form = parent::form($form, $form_state);

    if (HtmlExtra::getIsAjax()) {
      $form['intro'] = [
        '#weight' => -1000,
        '#markup' => '<h2>' . $form['#title'] . '</h2>',
      ];
    }

    return $form;
  }

  /**
   * {@inheritdoc}
   * Remove the delete action and add a cancel actionfor AJAX called forms.
   */
  protected function actions(array $form, FormStateInterface $form_state) {
    $actions = parent::actions($form, $form_state);
    $actions['submit']['#value'] = $this->t('Save settings');

    if (!$this->entity->isNew()) {
      $target_entity_type = $this->entityManager->getDefinition($this->entity->getTargetEntityTypeId());
      $route_parameters = [
        'field_config' => $this->entity->id(),
      ] + FieldUI::getRouteBundleParameter($target_entity_type, $this->entity->getTargetBundle());
      $url = new Url('entity.field_config.' . $target_entity_type->id() . '_field_delete_form', $route_parameters);

      if ($this->getRequest()->query->has('destination')) {
        $query = $url->getOption('query');
        $query['destination'] = $this->getRequest()->query->get('destination');
        $url->setOption('query', $query);
      }
      $actions['delete'] = array(
        '#type' => 'link',
        '#title' => $this->t('Delete'),
        '#url' => $url,
        '#access' => $this->entity->access('delete'),
        '#attributes' => array(
          'class' => array('button', 'button--danger'),
        ),
      );
    }
    if (HtmlExtra::getIsAjax()) {
      $form['intro'] = [
        '#weight' => -1000,
        '#markup' => '<h2>' . $form['#title'] . '</h2>',
      ];
      $selector = 'js-' . str_replace(['.', '_'], '-', $this->entity->id()) . '-edit-form';
      $actions['cancel'] = [
        '#markup' => '<a href="" class="button js-field-ui-toggle" data-field-ui-show=".action-links, .tableresponsive-toggle-columns, .js-field-ui-ajax-overview" data-field-ui-hide=".' . $selector . '">' . t('Cancel') . '</a>',
        '#weight' => 6,
      ];
      if (!$this->entity->isNew()) {
        // Remove delete button since there is already one in the overview table and
        // handling both would make the code more dificult.
        unset($actions['delete']);
      }
      $actions['submit']['#ajax'] = [
        'callback' => '::configAjaxEditFormSubmit',
      ];
    }

    return $actions;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    parent::submitForm($form, $form_state);
    if (!$form_state->hasAnyErrors() && HtmlExtra::getIsAjax()) {
      $form_state->setRebuild(TRUE);
    }
  }

  /**
   * Ajax callback for the "Save settings" button.
   *
   * This hides the form and shows back the action links and overview table.
   */
  public function configAjaxEditFormSubmit(array $form, FormStateInterface $form_state) {
    $response = new AjaxResponse();
    $selector = 'js-' . str_replace(['.', '_'], '-', $this->entity->id());
    $form_selector = $selector . '-edit-form';

    if ($form_state->hasAnyErrors()) {
      $build = [
        '#prefix' => '<div>',
        '#suffix' => '</div>',
        'messages' => [
          '#type' => 'status_messages',
        ],
        'form' => $form,
      ];
      $response->addCommand(new HtmlCommand(
        '.' . $form_selector,
        $build
      ));
    }
    else {
      // Update the field label
      $response->addCommand(new InvokeCommand(
        '.' . $selector . ' .js-field-label',
        'text',
        [$form_state->getValue('label')]
      ));
      $response->addCommand(new InvokeCommand(
        '.' . $form_selector,
        'addClass',
        ['js-field-ui-hidden']
      ));
      $response->addCommand(new InvokeCommand(
        '.action-links, .tableresponsive-toggle-columns, .js-field-ui-ajax-overview',
        'removeClass',
        ['js-field-ui-hidden']
      ));
      $response->addCommand(new HtmlCommand(
        '.' . $form_selector,
        $form
      ));
      // Show messages in a dialog.
      $response->addCommand(new OpenDialogCommand(
        '#field-ui-ajax-dialog',
        NULL,
        ['#type' => 'status_messages'],
        ['width' => '90%']
      ));
    }
    return $response;
  }

}
